
#include <windows.h>
#include <oal.h>
#include <bootarg.h>
#include <sci.h>

#ifdef DEBUG
//
// Debug zones
//
//#define ZONE_ERROR      DEBUGZONE(0)
//#define ZONE_WARNING    DEBUGZONE(1)
#define ZONE_FUNCTION   DEBUGZONE(2)
//#define ZONE_INIT       DEBUGZONE(3)
#define ZONE_PCMCIA     DEBUGZONE(4)
#define ZONE_IO         DEBUGZONE(5)
#define ZONE_MISC       DEBUGZONE(6)

#endif  // DEBUG

#define NOT_SUPPORT	0x0
#define VT686		0x1
#define VT8231		0x2
#define VT8233		0x3
#define VT8235		0x4
#define VT8237		0x5
#define VX700		0x6
#define VT8251		0x7
#define VT8237A		0x8
#define VT8237S		0x9
#define VX800		0xA	/*353 chip*/
#define VX855		0xB	/*409 chip*/
#define VX900		0xC	/*410 chip*/

/*power management device function0 (D0F0) bus id */
#define DEVID_VT0686	0x06861106
#define DEVID_VT8231	0x82311106
#define	DEVID_VT8233	0x31471106
#define	DEVID_VT8235	0x31771106
#define DEVID_VT8237	0x32271106
#define DEVID_VT8237A	0x33371106
#define DEVID_VT8237S	0x33721106
#define DEVID_VT8251	0x32871106
#define DEVID_VX700		0x83241106
#define DEVID_VX800		0x83531106
#define DEVID_VX855		0x84091106
#define DEVID_VX900		0x84101106	

#define CHECK_DEVICEID(xxx_id)  \
          ( (xxx_id == 0x30741106) \
          || (xxx_id == DEVID_VT8233) \
          || (xxx_id == DEVID_VT8235) \
          || (xxx_id == DEVID_VT8237) \
          || (xxx_id == DEVID_VX700) \
          || (xxx_id == DEVID_VT8251) \
          || (xxx_id == DEVID_VT8237A) \
          || (xxx_id == DEVID_VT8237S) \
          || (xxx_id == DEVID_VX800) \
          || (xxx_id == DEVID_VX855) \
		  || (xxx_id == DEVID_VX900) )
          
#define POWERBUTTON             0x01
#define EXTSMI                  0x02
#define GPIO4                   0x03
#define THERM                   0x04
#define NOTHING                 0x00

DWORD	SB686A_configid=0;
DWORD	g_dwConfigId;

static BOOL   bAutoConfig = TRUE;
static DWORD  IrqSetting,dwSLPBTN=1;

static DWORD  F0CFGID = 0,
			  F4CFGID = 0,
			  CHIP = NOT_SUPPORT,
			  ChipSetID = 0;


extern void NKSleep(DWORD);
extern BOOL OEMIoControl(DWORD ,LPVOID,DWORD,LPVOID ,DWORD ,LPDWORD);

static void ScreenOff(void);
static void ScreenOn(void);
static void SusToDisk(DWORD);
static BOOL GetIOBaseIRQ(DWORD *dwIoBase, DWORD *dwIRQ);

void ThrottleOn(DWORD dwIOBase)
{
    //CPU THROTTLING
    DEBUGMSG( ZONE_IO, (TEXT("SCIPWM:Throttling+\r\n")));

    _asm {
         mov  edx, dwIOBase        //processor control
         add  edx, 10h
         mov  ax, 0211h              //enable throttling & duty cycle <10%
         out  dx, ax

         mov  edx, dwIOBase        //processor level 2
         add  edx, 14h
         in   al,dx
    }
}


void ThrottleOff(DWORD dwIOBase)
{
    //CPU THROTTLING DISABLE
	DEBUGMSG( ZONE_IO, (TEXT("SCIPWM:Throttling off+\r\n")));

    _asm {
         mov  edx, dwIOBase        //processor control
         add  edx, 10h
         mov  al, 00h
         out  dx, al
    }
}


BOOL OALIoCtlSCIPWM( 
    UINT32 dwIoControlCode,
    PBYTE lpInBuf,
    UINT32 nInBufSize,
    PBYTE lpOutBuf,
    UINT32 nOutBufSize,
    UINT32* lpBytesReturned
) 
{
    
    BOOL bResult;
	DWORD dwIOBase,dwIRQ;
  
	DEBUGMSG(ZONE_IO, (TEXT("+OALIoCtlSCIPWM %X\r\n"), dwIoControlCode));

    switch (dwIoControlCode) {
		
		case IOCTL_HAL_POWEROFF:

			bResult = GetIOBaseIRQ(&dwIOBase, &dwIRQ);
	
			SusToDisk(dwIOBase);
			break;
		case IOCTL_HAL_SCREENOFF:
		       
			GetIOBaseIRQ(&dwIOBase , &dwIRQ);

			if (CHECK_DEVICEID(ChipSetID))
			{
				ScreenOff();
				ThrottleOn(dwIOBase);
			}
			else
				SusToDisk(dwIOBase);
			break;
		case IOCTL_HAL_SCREENON:
			GetIOBaseIRQ(&dwIOBase ,&dwIRQ);
			
			if (CHECK_DEVICEID(ChipSetID))
			{
				ScreenOn();
				ThrottleOff(dwIOBase);
			}
			else
				SusToDisk(dwIOBase);
			break;
		default:
			RETAILMSG(1, (TEXT("-OALIoCtlSCIPWM Error Command!\r\n")));
			return FALSE;

	}
	return TRUE;
}


void ScreenOn(void)
{
    DEBUGMSG( ZONE_IO, (TEXT("SCIPWM:turn on screen\r\n")));
    
    _asm {
        mov dx,3c0h   //reset flip-flop
        in  al,dx

        mov al,20h
        out dx,al     //turn on screen
        in  al,dx
        out dx,al


        mov dx,3c4h  //Enable CRTC R/W
        mov al,01h
        out dx,al

        inc dx      //port 3c5h
        in  al,dx
        and al,0d7h
        out dx,al
    }
}

static void ScreenOff(void)
{
    DEBUGMSG( ZONE_IO, (TEXT("SCIPWM:turn off screen\r\n")));
    
    _asm {
        mov dx,3c0h   //reset flip-flop
        in  al,dx

        xor al,al
        out dx,al     //turn off screen
        in  al,dx
        out dx,al

        mov dx,3c4h  //disable CRTC R/W
        mov al,01h
        out dx,al

        inc dx      //port 3c5h
        in  al,dx
        or  al,28h
        out dx,al
    }
}

static void SusToDisk(DWORD dwIOBase)
{
	int  i;

	DEBUGMSG(ZONE_IO, (TEXT("SCIPWM: SusToDisk(), STD\r\n")));

	//NIH-set SLP_EN and SMI Enable-start
	_asm
	{
		    ;; Global Control Enable (Offset 2B)
			mov    edx, dwIOBase
			add    edx, 02bh
			in      al, dx
			or      al, 04h
			out    dx, al
			jmp    short $+2

			;; Global Control Enable (Offset 2C)
			mov    edx, dwIOBase
			add    edx, 02Ch
			in	 al, dx
			or      al, 01h
			out    dx, al
			jmp    short $+2
	}
	//NIH-set SLP_EN and SMI Enable-end

	for (i = 0; i < 5; i++)
	{
		_asm
		{
			    ;;------Disable Reg.-------
				mov    eax, 0			
				;; Power Management Enable (Offset 5-4)
				mov    edx, dwIOBase
				add    edx, 04h
				out    dx, ax
				jmp    short $+2               ;delay time

				;; General Purpose Enable (Offset 25-22)
				mov    edx, dwIOBase
				add    edx, 22h
				out    dx, eax
				jmp    short $+2

				;; Primary Activity Detect Enable (Offset 37-34)
				mov    edx, dwIOBase
				add    edx, 034h
				out    dx, eax
				jmp    short $+2

				;;------Enable Reg.-------
				mov    eax, 0ffffffffh;
			    ;; Power Management Status (Offset 1-0)
				mov    edx, dwIOBase
				out    dx, ax
				jmp    short $+2

				mov    edx, dwIOBase
				add    edx, 020h
				out    dx, ax
				jmp    short $+2

				;; General Status (Offset 28, 29)
				mov    edx, dwIOBase
				add    edx, 028h
				out    dx, ax
				jmp    short $+2

				;; General Purpose Status (Offset 30-33)
				mov    edx, dwIOBase
				add    edx, 030h
				out    dx, eax
				jmp    short $+2

				mov    eax, 0h
				;; Processor & PCI Bus Control (Offset 11-10)
				mov    edx, dwIOBase
				add    edx, 010h
				out    dx, ax
				jmp    short $+2

		}


		if (ChipSetID == 0x30401106)
		{
			// VT82C586
			_asm
			{
				mov    edx, dwIOBase
				add    edx, 05h
				mov    al, 20h
				out    dx, al
			}

		}
		else
		{ 
			// Suspend to Disk, set to 28h
			_asm
			{
				mov    edx, dwIOBase
				;; Power Management Control (Offset 5)
				add    edx, 05h
				mov    al, 28h
				out    dx, al
			}

		}

		NKSleep(200);
	}

}

static BOOL GetIOBaseIRQ(DWORD *pdwGetIOBase, DWORD *pdwGetIRQ)
{
    DWORD		portBase, ACPI_Irq;
    BOOL		bFoundIt;
    DWORD		configid, deviceid;
    WORD		GenPurEV_Sts;
   
    bFoundIt = FALSE;				   
    // check the device chipset info,and get the device offset address
    // of power management.
    for ( configid = 0x80000000; configid < 0x80010800; configid += 0x100 )
    {
        _asm
        {
            mov  eax, configid
            mov  edx, 0cf8h
            out  dx, eax

            mov  edx, 0cfch
            in   eax, dx
            mov  deviceid, eax
        }
         
        if (deviceid == DEVID_VT0686) {
           SB686A_configid = configid;
           F0CFGID = configid;
           CHIP = VT686;
        }
        else if(deviceid == DEVID_VT8231)
        {
           F0CFGID = configid;
           CHIP = VT8231;
        }
    
       	if (CHECK_DEVICEID(deviceid))
        {
            ChipSetID = deviceid;
            F0CFGID = F4CFGID = configid;
            // Enable ACPI IO Address, and Get ACPI IO Base, and ACPI Irq
            if (deviceid == DEVID_VT8233)
            {
            	CHIP = VT8233;
            }
            else if (deviceid == DEVID_VT8235)
            {
            	CHIP = VT8235;
            }
            else if (deviceid == DEVID_VT8237)
            {
            	CHIP = VT8237;
            }
            else if (deviceid == DEVID_VT8251)
            {
            	CHIP = VT8251;
            }
            else if (deviceid == DEVID_VX700)
            {
            	CHIP = VX700;
            }
            else if (deviceid == DEVID_VT8237A)
            {
            	CHIP = VT8237A;
            }
            else if(deviceid == DEVID_VT8237S)
            {
            	CHIP = VT8237S;
            }
            else if(deviceid == DEVID_VX800)
            {
            	CHIP = VX800;
            }
            else if(deviceid == DEVID_VX855)
            {	
            	CHIP = VX855;	
            }
	        else if (deviceid == DEVID_VX900)
	       {
            	CHIP = VX900;
	        }
      
            _asm
            {
                  mov    eax, configid
                  add    ax, 080h
                  mov    edx, 0cf8h
                  out    dx, eax
            
                  mov    edx, 0cfch
                  in     eax, dx
                  mov    ACPI_Irq, eax              ; ACPI_IRQs value : Offset80-83h bits
                  or     eax, 00008000h             ; Enable ACPI I/O (81h-7bits)
                  out    dx, eax
            
                  mov    eax, configid
                  add    eax, 088h					; ACPI IOBase Offset address
                  mov    edx, 0cf8h
                  out    dx, eax
            
                  mov    edx, 0cfch
                  in     eax, dx
                  and    eax, 0fffffffeh			; Mask the 0-6bits
                  mov    portBase, eax 
            }
           
           //Read out ACPI_IRQ 
           ACPI_Irq = ACPI_Irq >> 16;
           ACPI_Irq = ACPI_Irq & 0x0000000f;
	       if(ACPI_Irq == 0)
	       {
	          bAutoConfig = 0;
	          IrqSetting = 9; 
	       }
            
            if (!bAutoConfig)//Write ACPI IRQ to registry
            {
              WORD wtemp = 1 << IrqSetting;
              ACPI_Irq = IrqSetting << 16;
              _asm
              {
                  mov    eax, configid
                  add    ax, 080h
                  mov    edx, 0cf8h
                  out    dx, eax
              
                  mov    edx, 0cfch
                  in     eax, dx
                  and    eax, 0fff0ffffh
                  or     eax, ACPI_Irq
                  out    dx, eax
              
                  mov    dx, 04d0h
                  in     ax, dx
                  or     ax, wtemp
                  out    dx, ax
              }
              
              //RETAILMSG(1, (TEXT("SCIPWM: bAutoConfig=FALSE, Assign SCI IRQ:0x%x\r\n"), IrqSetting));
			  
              bAutoConfig = 1;
	          ACPI_Irq = ACPI_Irq >> 16;
              ACPI_Irq = ACPI_Irq & 0x0000000f;
            }
	  
            if (ACPI_Irq == 0)
            {
		       RETAILMSG(1,(TEXT("SCIPWM: Can not Assign IRQ for ACPI\r\n")));
		       return FALSE;
            }

           bFoundIt = TRUE;           
           RETAILMSG(ZONE_INIT, (TEXT("SCIPWM: Irq 0x%X, IOBase 0x%X\r\n"), ACPI_Irq, portBase));
           break;		     
        }    
		
        //////////////////////////////////////////////////////////       
		if ((deviceid == 0x30571106)
		  || (deviceid == 0x30501106)
		  || (deviceid == 0x30401106)
		  || (deviceid == 0x82351106))
		{
		   
		   if(deviceid == 0x30571106)
		   {
		      F4CFGID = configid;              
		   }
		   else if(deviceid == 0x82351106)//
		   { 	
		      F4CFGID = configid;
		   }
		   
		   ChipSetID = deviceid;
		   
		   // Enable ACPI IO Address, and Get ACPI IO Base, and ACPI Irq
		   if (dwSLPBTN == 1)
		   {
		      _asm
		      {
		          mov    eax, configid
		          add    ax, 040h
		          mov    edx, 0cf8h
		          out    dx, eax

		          mov    edx, 0cfch
		          in     eax, dx
		          mov    ACPI_Irq, eax                           
		          ;or     eax, 00008000h
		          or     eax, 00008040h      ;Enable sleep button
		          out    dx, eax
		      }
		   }
		   else
		   {
		      _asm
		      {
		          mov    eax, configid
		          add    ax, 040h
		          mov    edx, 0cf8h
		          out    dx, eax

		          mov    edx, 0cfch
		          in     eax, dx
		          mov    ACPI_Irq, eax                           
		          or     eax, 00008000h                  
		          out    dx, eax
		      }
		   }
		   
		   _asm
		   	{
		       mov    eax, configid
		       add    eax, 048h
		       mov    edx, 0cf8h
		       out    dx, eax

		       mov    edx, 0cfch
		       in     eax, dx
		       and    eax, 0fffffffeh
		       mov    portBase, eax
		   }

		   // For VT82C586B Rev. 3040E
		   if ( (deviceid == 0x30401106)
		    && ( (portBase == 0) || (portBase == 0xfffffffe) ) )
		   {
		      _asm
				{
		          mov    eax, configid
		          add    eax, 048h
		          mov    edx, 0cf8h
		          out    dx, eax

		          mov    edx, 0cfch
		          in     eax, dx
		          and    eax, 0fffffffeh
		          mov    portBase, eax
				}
		   }
		    
		   if (!bAutoConfig)
		   {
		      WORD wtemp = 1 << IrqSetting;
		      ACPI_Irq = IrqSetting << 16;
		      _asm
		      {
		          mov    eax, configid
		          add    ax, 040h
		          mov    edx, 0cf8h
		          out    dx, eax

		          mov    edx, 0cfch
		          in     eax, dx
		          and    eax, 0fff0ffffh
		          or     eax, ACPI_Irq
		          out    dx, eax

		          mov    dx, 04d0h
		          in     ax, dx
		          or     ax, wtemp
		          out    dx, ax
				}
				RETAILMSG(1, (TEXT("SCIPWM:Assign SCI IRQ:0x%x\r\n"), IrqSetting));
			}
	 
			ACPI_Irq = ACPI_Irq >> 16;
			ACPI_Irq = ACPI_Irq & 0x0000000f;

			if (ACPI_Irq == 0)
			{
				DEBUGMSG(1, (TEXT("SCIPWM: Please Assign Irq for ACPI\r\n")));
		        ACPI_Irq = 9;
				//return FALSE;
			}

			bFoundIt = TRUE;
			RETAILMSG(ZONE_INIT,(TEXT("SCIPWM: Irq 0x%X, IOBase 0x%X\r\n"), ACPI_Irq, portBase));
			break;
        }

    }
    
    if (!bFoundIt) return bFoundIt;
    
    *pdwGetIOBase	= portBase;   
    *pdwGetIRQ		= ACPI_Irq;
    g_dwConfigId	= configid;
    
    // Enable SCI
    _asm
    {
        mov  edx, portBase
        add  edx, 04h
        in   al, dx
        or   al, 01h            ; Enable SCI
        out  dx, al

        mov  edx, portBase
        add  edx, 24h
        in   ax, dx
        ;and  ax, 0fffeh
        and  ax, 0ff00h
        out  dx, al             ; Disable ALL GPIO to generate SMI

        mov  edx, portBase      ;clear current status offset 0h-1h & 20h-21h
        mov  ax,0xffff          ;
        out  dx,ax
        add  dx,20h
        out  dx,ax
    }
    
    //Check current general purpose status
    do {
		_asm
		{
			mov  edx, portBase
			add  edx, 20h
			in   ax, dx
			mov  GenPurEV_Sts, ax    ;GenPurEV_Sts : Offset21-20 bits
			out  dx, ax
		}
		DEBUGMSG(ZONE_IO, (TEXT("SCIPWM: General Purpose status:%04XH\r\n"),GenPurEV_Sts));
    } while(GenPurEV_Sts);
    
    
    if (dwSLPBTN == 1)
    {
       _asm {
          mov  edx, portBase  ;enable pwr button
          add  dx,3h
          mov  al,0x3 
          out  dx,al
       }
    }
    else
    {
       _asm {
          mov  edx, portBase  ;enable pwr button
          add  dx,3h
          mov  al,0x1
          out  dx,al
       }
    }
    

    _asm {
            mov  edx, portBase
            add  edx, 22h
            in   ax, dx
            or   ax, 0401h          ;THERM|EXT_SMI
            out  dx, ax
        }

    return bFoundIt;
}




